#! /bin/bash

# -----------------------------------------
#   WombatDialer database clean-up script
# -----------------------------------------
#
# Use:
#  ./cleanup_wbt.sh -uroot -pabcd -dwombatdb -t2016-01-01
#
# Cleans up database "wombatdb" (user: root passw: abcd)
# of all runs terminated before "2016-01-01" that belong
# to hidden campaigns.
#
# Pls refer to docs and MAKE A BACKUP FIRST!
#

while getopts u:d:p:t:m: option
do
 case "${option}"
 in
 u) DBUSER=${OPTARG};;
 d) DATABASE=${OPTARG};;
 p) PASS=${OPTARG};;
 t) OLDERTHAN=$OPTARG;;
 m) MODE=${OPTARG};;
 esac
done

if [ -z "$DBUSER" ]
  then
    echo "No username was supplied (-u)"
    exit 1
fi

if [ -z "$DATABASE" ]
  then
    echo "No database name was supplied (-d)"
    exit 1
fi

if [ -z "$PASS" ]
  then
    echo "No password was supplied (-p)"
    exit 1
fi


if ! [[ $OLDERTHAN =~ ^[0-9]{4}-[0-9]{2}-[0-9]{2}$ ]]
  then
    echo "You must insert a valid date with the following format: yyyy-mm-dd (-t)"
    exit 1
fi


SQL_CLEANLISTS_WHERE="
WHERE isHidden = 1 
  AND listId NOT IN (
      SELECT DISTINCT listID FROM (
             SELECT DISTINCT cl listID FROM campaigns_cl 
             UNION 
             SELECT DISTINCT list listID FROM hop_lists
      ) X1 
  )
"
	
#FROM_HOPPER="
#     SELECT hopId
#     FROM hopper 
#     WHERE completed < '$OLDERTHAN'
#"

# https://stackoverflow.com/questions/45494/mysql-error-1093-cant-specify-target-table-for-update-in-from-clause
# devo usare hopper.* se no mi globba i files con il solo *
FROM_HOPPER="
    SELECT hopId
      FROM (SELECT hopper.* FROM hopper) H
      JOIN campaigns C on H.campaign = C.campaignId
     WHERE (H.completed < '$OLDERTHAN')
       AND C.pace = 'HIDDEN'
"


SQL_CMPS="
     SELECT campaignId 
       FROM (SELECT campaigns.* FROM campaigns) C
      WHERE campaignId NOT IN (SELECT campaign FROM hopper)
"


function sql_displ {
	COMMENT=$1
	QUERY=$2
	#QUERY="${QUERY_NC}; SELECT ROW_COUNT() as rows_affected;"
	echo ""
	echo "** ${COMMENT} **"
	echo "----------------"
	#echo "QQ: $QUERY"
	export MYSQL_PWD=$PASS
	echo $QUERY | mysql -u$DBUSER $DATABASE 
}

function sql {
	COMMENT=$1
	QUERY_NC=$2
	QUERY="${QUERY_NC}; SELECT ROW_COUNT() as rows_affected;"
	sql_displ "${COMMENT}" "${QUERY}"	
}


function optimize {
	TBL=$1
	sql_displ "Optimizing table: $TBL" "OPTIMIZE TABLE $TBL;"
}

function list_hopper_elements {

	sql_displ "Hopper elements to be removed" "
	     SELECT hopId, hopName, campaign
	     FROM hopper
	     WHERE hopId IN ( $FROM_HOPPER );    
	"

}

function call_lists_to_delete {

	#; conta le liste da cancellare

	sql "Call Lists that are no more referenced and will be deleted" "

	SELECT CL.listId, CL.name, 
	       (SELECT COUNT(*) FROM call_records R WHERE R.listId = CL.listId ) AS records_to_delete
	FROM call_lists CL  $SQL_CLEANLISTS_WHERE;
	"

}

function remove_elements {

	# Logica:
	# Cancello dall'hopper tutti i runs
	# che sono piu' vecchi di MAXDATE e che 
	# appartengono a campagne HIDDEN 

	sql "Remove elements from hop_lists" "
	    DELETE FROM hop_lists
	    WHERE hopListId IN ( $FROM_HOPPER );"

	sql "Remove elements from hop_calls" "
		DELETE FROM hop_calls
	     where hopId IN ( $FROM_HOPPER );
	"

	sql "Remove elements from campaign_retries" "
		DELETE FROM campaigns_retries
		where hopId IN ( $FROM_HOPPER );
	"

	sql "Remove selected elements from hopper" " 
		 DELETE FROM hopper 
	     WHERE hopId IN ( $FROM_HOPPER );
	"

	# Tutte le campagne HIDDEN

	sql "campaigns_cl" "
		DELETE FROM campaigns_cl WHERE campaignId IN ( $SQL_CMPS )
	"


	sql "campaigns_eps" "
		DELETE FROM campaigns_eps WHERE campaignId IN ( $SQL_CMPS )
	"

	sql "campaigns_ohs" "
		DELETE FROM campaigns_ohs WHERE campaignId IN ( $SQL_CMPS )
	"

	sql "campaigns_tks" "
		DELETE FROM campaigns_tks WHERE campaignId IN ( $SQL_CMPS )
	"

	sql "Unused campaigns" "
	DELETE FROM campaigns WHERE campaignId IN ( $SQL_CMPS )
	"

	# DELETE LOG FACTS AND LOG DATA
	sql "Remove log facts of deleted campaigns" "
	DELETE FROM log_f 
	WHERE dimensions IN (
	SELECT id 
	  FROM log_d 
	  WHERE campId NOT IN (SELECT campaignId FROM campaigns));
	"


	sql "Remove campaign from logs" "
	DELETE FROM log_d WHERE campId NOT IN (SELECT campaignId FROM campaigns);
	"

	sql "Syslog" "
	DELETE FROM arch_syslog WHERE sys_dt_creazione < '$OLDERTHAN';
	"



	#; conta le liste da cancellare

	call_lists_to_delete

	sql "Call records" "
	DELETE FROM call_records WHERE listId 
	IN ( SELECT listId from call_lists  $SQL_CLEANLISTS_WHERE );"


	sql "Call record attributes (takes a bit)" "
	 DELETE FROM call_record_attributes
	 WHERE call_record_id NOT IN (SELECT callId FROM call_records);
	"

	sql "Call lists" "
	DELETE FROM call_lists $SQL_CLEANLISTS_WHERE;
	"
}

function optimize_tables {
		# optimize
		tables="arch_classes
		arch_lic
		arch_syslog
		arch_users
		astservers
		call_lists
		call_logs
		call_logs_backup
		call_record_attributes
		call_records
		campaigns
		campaigns_cl
		campaigns_eps
		campaigns_oh
		campaigns_ohs
		campaigns_retries
		campaigns_tks
		dbversion
		disp_rules
		endpoints
		hop_calls
		hop_lists
		hopper
		log_d
		log_f
		open_hr
		open_hr_rules
		rsch_rules
		trunks"

		for tbl in $tables
		do
			optimize $tbl
		done
}


case "${MODE}" in
list) 
	list_hopper_elements;;
delete)
	remove_elements;;
optimize)
	optimize_tables;;
preview)
	list_hopper_elements
	call_lists_to_delete;;
""|all)
	list_hopper_elements
	read -p 'These Hopper elements will be removed, do you wish to continue? [y/n]' -n 1 -r
	echo    # (optional) move to a new line
	if [[ $REPLY =~ ^[Yy]$ ]]
	then
		
		remove_elements
		
		read -p 'Dependent objects cleanup completed. Optimize tables? [y/n]' -n 1 -r
		echo    # (optional) move to a new line
		if [[ $REPLY =~ ^[Yy]$ ]]
		then
		
			optimize_tables

		fi

	fi;;
*)
	echo 'selected mode (-m) is invalid, available modes are: list, delete, optimize, preview, all. The default mode is "all"'
esac


